//@ts-check
"use strict";

/**
@typedef Spritesheet
@type {{
  anchor: PIXI.Point;
  animations: { [key: string]: PIXI.AnimatedSprite.FrameObject[] }
}}
*/

const spritesheetsByName = {};

/**
 * @param {string} name
 * @param {number} textureVariants
 * @returns {Spritesheet}
 * */
function getSpritesheet(name, textureVariants = null) {
  const textureName = textureVariants == null ? name : `${name}${randInt(1, textureVariants)}`;

  if (spritesheetsByName[textureName]) {
    return spritesheetsByName[textureName];
  }

  const sheetTexture = tex(textureName);
  const spritesheetJson = jsonRes(name + ".sprite");

  const { width, height } = spritesheetJson;
  const framesPerRow = sheetTexture.width / width;

  /** @type {{ [name: string]: PIXI.AnimatedSprite.FrameObject[] }} */
  const animations = {};

  for (const [animName, animFrames] of Object.entries(spritesheetJson.anims)) {
    const frames = [];

    if (Array.isArray(animFrames)) {
      for (const frame of animFrames) {
        const texture = new PIXI.Texture(sheetTexture.baseTexture, new PIXI.Rectangle((frame.index % framesPerRow) * width, Math.floor(frame.index / framesPerRow) * height, width, height));
        frames.push({ texture, time: frame.duration });
      }
    } else {
      for (let i = animFrames.start; i <= animFrames.end; i++) {
        const texture = new PIXI.Texture(sheetTexture.baseTexture, new PIXI.Rectangle((i % framesPerRow) * width, Math.floor(i / framesPerRow) * height, width, height));
        frames.push({ texture, time: animFrames.duration });
      }
    }

    animations[animName] = frames;
  }

  const spritesheet = spritesheetsByName[name] = { animations, anchor: new PIXI.Point(spritesheetJson.anchor.x, spritesheetJson.anchor.y) };
  return spritesheet;
}

/**
 * 
 * @param {PIXI.AnimatedSprite} sprite s
 * @param {any} animation 
 */
function setAnimation(sprite, animation) {
  //@ts-ignore
  if (sprite._woodpunkAnim == animation) return;
  //@ts-ignore
  sprite._woodpunkAnim = animation;

  sprite.textures = animation;
  sprite.play();

}
